import 'package:_private_core/utils/device_infos.dart';
import 'package:_private_core/_private_core.dart';
import 'package:_private_core/utils/utils.dart';
import '../resources.dart';

class UpdateInfoResult {
  bool status;
  String? msg;

  UpdateInfoResult({required this.status, this.msg});
}

class CheckNewEmailProResult {
  int? statusCode;
  String? msg;
  String? thumbnail;
  String? displayName;
  dynamic roleId;

  CheckNewEmailProResult(
      {required this.statusCode,
      this.msg,
      this.displayName,
      this.roleId,
      this.thumbnail});
}

class AuthRepo {
  AuthRepo._();

  static AuthRepo? _instance;

  factory AuthRepo([AppApi? api]) {
    _instance ??= AuthRepo._();
    _instance!._api = api ?? AppApiImp();
    return _instance!;
  }

  late AppApi _api;

  // ///Phone on MOBILE
  // Future<void> verifyPhoneNumber({
  //   required String phoneNumber,
  //   required PhoneVerificationCompleted verificationCompleted,
  //   required PhoneVerificationFailed verificationFailed,
  //   required PhoneCodeSent codeSent,
  //   required PhoneCodeAutoRetrievalTimeout codeAutoRetrievalTimeout,
  // }) {
  //   return FirebaseAuth.instance.verifyPhoneNumber(
  //       phoneNumber: phoneNumber,
  //       verificationCompleted: verificationCompleted,
  //       verificationFailed: verificationFailed,
  //       codeSent: codeSent,
  //       timeout: const Duration(seconds: 90),
  //       codeAutoRetrievalTimeout: codeAutoRetrievalTimeout);
  // }

  // ///Phone on WEB
  // Future<ConfirmationResult?> signInWithPhoneNumber(String phone) async {
  //   try {
  //     ConfirmationResult confirmationResult =
  //         await FirebaseAuth.instance.signInWithPhoneNumber(phone);
  //     return confirmationResult;
  //   } catch (e) {
  //     appDebugPrint(e);
  //   }
  // }

  // Future<UserCredential> confirmationResult(
  //         ConfirmationResult confirmationResult, String code) =>
  //     confirmationResult.confirm(code);

  // ///=============
  // Future<UserCredential> signInWithCredential(AuthCredential credential) =>
  //     FirebaseAuth.instance.signInWithCredential(credential);

  // Future<UserCredential> signInWithEmailPassword(
  //         String email, String password) =>
  //     FirebaseAuth.instance
  //         .signInWithEmailAndPassword(email: email, password: password);

  /// API
  Future<bool> checkNewPhoneFromPro(phone) async {
    NetworkResponse response = await _api.checkNewPhoneFromPro(phone);
    return response.data ?? false;
  }

  Future<bool> checkNewPhoneFromUser(phone) async {
    NetworkResponse response = await _api.checkNewPhoneFromUser(phone);
    return response.data ?? false;
  }

  Future<UserModel?> checkExistPhoneFromUser(phone) async {
    NetworkResponse response = await _api.checkExistPhoneFromUser(phone);
    return response.data;
  }

  Future<bool> updateNewUserPasswordByFirebase(params) async {
    NetworkResponse response =
        await _api.updateNewUserPasswordByFirebase(params);
    return response.data ?? false;
  }

  Future<Map> _addParams(
    params, {
    nnotificationToken = false,
    ndeviceInfos = true,
  }) async {
    Map realParams = Map.from(params);
    realParams.addAll({
      'allowPrivacyTypeId': 1,
      'platformId': valuePlatformByConfig(),
      'dashboard': 'oowoo',
    });
    if (isPlatformStateInitialized && ndeviceInfos) {
      realParams.addAll({'deviceInfos': deviceData});
    }

    if (nnotificationToken) {
      String? fcmToken;
      try {
        fcmToken = await coreOptions.getTokenFCM?.call();
      } catch (e) {
        appDebugPrint(e);
      }
      realParams.addAll({
        "notificationToken": fcmToken,
        "notificationProvider": "fcm",
      });
    }
    return realParams;
  }

  Future<NetworkResponse> loginUserByPhone(params) async {
    return await _api
        .loginUserByPhone(await _addParams(params, nnotificationToken: true));
  }

  Future<Token?> loginUserByEmail(params) async {
    NetworkResponse response = await _api
        .loginUserByEmail(await _addParams(params, nnotificationToken: true));
    return response.data;
  }

  Future<Token?> createOrLoginPro(params) async {
    NetworkResponse response = await _api
        .createOrLoginPro(await _addParams(params, nnotificationToken: true));
    return response.data;
  }

  Future<Token?> createNewUser(params) async {
    NetworkResponse response = await _api
        .createNewUser(await _addParams(params, nnotificationToken: true));
    return response.data;
  }

  Future<Token?> subRegisterUser() async {
    NetworkResponse response = await _api.subRegisterUser(await _addParams({}));
    return response.data;
  }

  Future<ProInfo?> getProInfo() async {
    NetworkResponse response = await _api.getProInfo();
    return response.data;
  }

  Future<UserModel?> getUserInfo() async {
    NetworkResponse response = await _api.getUserInfo();
    return response.data;
  }

  Future<UpdateInfoResult> updatePro(params) async {
    NetworkResponse response = await _api.updatePro(params);
    return response.data ?? UpdateInfoResult(status: false);
  }

  Future<UpdateInfoResult> updateUser(params) async {
    NetworkResponse response = await _api.updateUser(params);
    return response.data ?? UpdateInfoResult(status: false);
  }

  Future<UpdateInfoResult> updateProPwd(
      {required String oldPassword, required String newPassword}) async {
    NetworkResponse response =
        await _api.updateProPwd(oldPassword, newPassword);
    return response.data ?? UpdateInfoResult(status: false);
  }

  Future<UpdateInfoResult> updateUserPwd(
      {required String oldPassword, required String newPassword}) async {
    NetworkResponse response =
        await _api.updateUserPwd(oldPassword, newPassword);
    return response.data ?? UpdateInfoResult(status: false);
  }

  Future<String?> setThumbnailForPro(image, filePath) async {
    NetworkResponse response = await _api.setThumbnailForPro(image, filePath);
    return response.data;
  }

  Future<String?> setThumbnailForUser(image, filePath) async {
    NetworkResponse response = await _api.setThumbnailForUser(image, filePath);
    return response.data;
  }

  Future<Token?> refreshUserToken(String refresh) async {
    NetworkResponse response = await _api.refreshUserToken(refresh);
    return response.data;
  }

  Future<Token?> refreshProToken(String refresh) async {
    NetworkResponse response = await _api.refreshProToken(refresh);
    return response.data;
  }

  Future<bool> addNotificationToken([fcmToken, provider]) async {
    try {
      if (fcmToken == null) {
        fcmToken = await coreOptions.getTokenFCM?.call();
        provider = 'fcm';
      }
      Map<String, dynamic> params = {
        'fcmToken': fcmToken,
        "dashboard": "oowoo",
        "provider": provider
      };
      NetworkResponse response = await _api.addNotificationToken(params);
      return response.data ?? false;
    } catch (e) {
      appDebugPrint('addNotificationToken: $e');
      return false;
    }
  }

  Future<bool> logoutPro([fcmToken]) async {
    NetworkResponse response =
        await _api.logoutPro(await _addParams({'fcmToken': fcmToken}));
    return response.data ?? false;
  }

  Future<bool> logoutUser([fcmToken]) async {
    NetworkResponse response = await _api.logoutUser(
        await _addParams({'fcmToken': fcmToken}, ndeviceInfos: false));
    return response.data ?? false;
  }

  Future<UpdateInfoResult> checkUpdateEmailFromUser(email) async {
    NetworkResponse response = await _api.checkUpdateEmailFromUser(
        await _addParams({'email': email}, ndeviceInfos: false));
    return response.data ?? UpdateInfoResult(status: false);
  }

  Future<NetworkResponse> confirmUpdateUserEmailCode(email, code, phone) async {
    return await _api.confirmUpdateUserEmailCode(email, code, phone);
  }

  Future<bool> updateUserEmail(token) async {
    NetworkResponse response = await _api.updateUserEmail(token);
    return response.data ?? false;
  }

  Future<UpdateInfoResult> checkUpdateEmailFromPro(email) async {
    NetworkResponse response =
        await _api.checkUpdateEmailFromPro(await _addParams({'email': email}));
    return response.data ?? UpdateInfoResult(status: false);
  }

  Future<NetworkResponse> confirmUpdateProEmailCode(email, code) async {
    return await _api.confirmUpdateProEmailCode(email, code);
  }

  Future<bool> updateProEmail(token) async {
    NetworkResponse response = await _api.updateProEmail(token);
    return response.data ?? false;
  }

  Future<NetworkResponse> loginPro(params) async {
    return await _api
        .loginPro(await _addParams(params, nnotificationToken: true));
  }

  Future<NetworkResponse> loginUserByPhoneAndFirebase(params) async {
    return await _api.loginUserByPhoneAndFirebase(
        await _addParams(params, nnotificationToken: true));
  }

  Future<NetworkResponse> loginProByPhoneAndFirebase(params) async {
    return await _api.loginProByPhoneAndFirebase(
        await _addParams(params, nnotificationToken: true));
  }

  Future<Token?> loginProCrmByQrCode(params) async {
    NetworkResponse response = await _api.loginProCrmByQrCode(
        await _addParams(params, nnotificationToken: true));
    return response.data;
  }

  Future<Token?> createOrLoginUser(params) async {
    NetworkResponse response = await _api
        .createOrLoginUser(await _addParams(params, nnotificationToken: true));
    return response.data;
  }

  Future<CheckNewEmailProResult?> checkEmailPro(email) async {
    NetworkResponse response = await _api.checkEmailPro({"email": email});
    return response.data;
  }

  Future<CheckNewEmailProResult?> checkNewProEmail(email) async {
    NetworkResponse response = await _api.checkNewProEmail({
      "email": email,
      'allowPrivacyTypeId': 1,
      'platformId': valuePlatformByConfig(),
      'dashboard': 'oowoo'
    });
    return response.data;
  }

  Future<CheckNewEmailProResult?> checkIfEmailExist(email) async {
    NetworkResponse response = await _api.checkIfEmailExist({
      "email": email,
      'allowPrivacyTypeId': 1,
      'platformId': valuePlatformByConfig(),
      'dashboard': 'oowoo'
    });
    return response.data;
  }

  Future<ConfirmOtp?> checkNewEmailUserEmailConfirm({email, code}) async {
    NetworkResponse response = await _api.checkNewEmailUserEmailConfirm({
      "email": email,
      "code": code,
    });
    return response.data;
  }

  Future<bool> checkLoginProByEmailOTP(email) async {
    NetworkResponse response = await _api.checkLoginProByEmailOTP({
      "email": email,
      'platformId': valuePlatformByConfig(),
    });
    return response.data ?? false;
  }

  Future loginProByEmailOTPConfirm(params) async {
    return await _api.loginProByEmailOTPConfirm(
        await _addParams(params, nnotificationToken: true));
  }

  Future<ConfirmOtp?> confirmNewProCode({email, code}) async {
    NetworkResponse response = await _api.confirmNewProCode({
      "email": email,
      "code": code,
    });
    return response.data;
  }

  Future<ConfirmOtp?> confirmResetProPasswordCode({email, code}) async {
    NetworkResponse response = await _api.confirmResetProPasswordCode({
      "email": email,
      "code": code,
    });
    return response.data;
  }

  Future<bool> updateNewProPasswordByToken({password, token}) async {
    NetworkResponse response = await _api.updateNewProPasswordByToken({
      "password": password,
      "token": token,
    });
    return response.data;
  }

  Future<Token?> createNewPro(params) async {
    NetworkResponse response = await _api
        .createNewPro(await _addParams(params, nnotificationToken: true));
    return response.data;
  }

  Future<List<MainCat>> getCategoriesList() async {
    NetworkResponse response = await _api.getCategoriesList();
    return response.data ?? [];
  }

  Future<bool> checkExistEmailFromPro(email) async {
    NetworkResponse response = await _api.checkExistEmailFromPro({
      "email": email,
    });
    return response.data ?? false;
  }

  Future<bool> resetProPassword(email) async {
    NetworkResponse response =
        await _api.resetProPassword(await _addParams({"email": email}));
    return response.data ?? false;
  }

  Future<ProInfo?> getProInfoByUniqueId(uniqueId) async {
    NetworkResponse response = await _api.getProInfoByUniqueId(uniqueId);
    return response.data;
  }

  Future<UserModel?> getUserInfoByUniqueId(uniqueId) async {
    NetworkResponse response = await _api.getUserInfoByUniqueId(uniqueId);
    return response.data;
  }

  Future<UserModel?> getUserInfoByFirebaseId(firebaseId) async {
    NetworkResponse response = await _api.getUserInfoByFirebaseId(firebaseId);
    return response.data;
  }

  Future<ProsModel?> getProInfoByFirebaseId(firebaseId) async {
    NetworkResponse response = await _api.getProInfoByFirebaseId(firebaseId);
    return response.data;
  }

  Future<OrgProInfo?> getAssetsForProFromUniversalDashboard(proUniqueId) async {
    NetworkResponse response =
        await _api.getAssetsForProFromUniversalDashboard(proUniqueId);
    return response.data;
  }

  Future loginTerminal({required params}) async {
    return await _api.loginTerminal(params);
  }

  Future<String?> setKisblePublicThumbnailForPro(bytes, path, proId) async {
    NetworkResponse response =
        await _api.setKisblePublicThumbnailForPro(bytes, path, proId);
    return response.data;
  }

  Future<String?> setThumbnailForProFromSA(bytes, path, proId) async {
    NetworkResponse response =
        await _api.setThumbnailForProFromSA(bytes, path, proId);
    return response.data;
  }

  Future<String?> setThumbnailForUserFromSA(bytes, path, proId) async {
    NetworkResponse response =
        await _api.setThumbnailForUserFromSA(bytes, path, proId);
    return response.data;
  }

  Future<UpdateInfoResult> updateProFromSA(params) async {
    NetworkResponse response = await _api.updateProFromSA(params);
    return response.data ?? UpdateInfoResult(status: false);
  }

  Future<UpdateInfoResult> updateUserFromSA(params) async {
    NetworkResponse response = await _api.updateUserFromSA(params);
    return response.data ?? UpdateInfoResult(status: false);
  }

  Future<List<SubscriptionState>> getSubscriptionState(params) async {
    NetworkResponse response = await _api.getSubscriptionState(params);
    return response.data ?? [];
  }
}
